@@ -21,13 +21,12 @@
 
 /* LangSys */
 
-static FT_Error  Load_LangSys( HB_LangSys*  ls,
-			       FT_Stream     stream )
+static HB_Error  Load_LangSys( HB_LangSys*  ls,
+			       HB_Stream     stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
-  FT_UShort  n, count;
-  FT_UShort* fi;
+  HB_Error   error;
+  HB_UShort  n, count;
+  HB_UShort* fi;
 
 
   if ( ACCESS_Frame( 6L ) )
@@ -41,7 +40,7 @@ static FT_Error  Load_LangSys( HB_LangSys*  ls,
 
   ls->FeatureIndex = NULL;
 
-  if ( ALLOC_ARRAY( ls->FeatureIndex, count, FT_UShort ) )
+  if ( ALLOC_ARRAY( ls->FeatureIndex, count, HB_UShort ) )
     return error;
 
   if ( ACCESS_Frame( count * 2L ) )
@@ -57,12 +56,11 @@ static FT_Error  Load_LangSys( HB_LangSys*  ls,
 
   FORGET_Frame();
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 }
 
 
-static void  Free_LangSys( HB_LangSys*  ls,
-			   FT_Memory     memory )
+static void  Free_LangSys( HB_LangSys*  ls )
 {
   FREE( ls->FeatureIndex );
 }
@@ -70,13 +68,12 @@ static void  Free_LangSys( HB_LangSys*  ls,
 
 /* Script */
 
-static FT_Error  Load_Script( HB_Script*  s,
-			      FT_Stream    stream )
+static HB_Error  Load_Script( HB_ScriptTable*  s,
+			      HB_Stream    stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
-  FT_UShort  n, m, count;
-  FT_ULong   cur_offset, new_offset, base_offset;
+  HB_Error   error;
+  HB_UShort  n, m, count;
+  HB_UInt   cur_offset, new_offset, base_offset;
 
   HB_LangSysRecord*  lsr;
 
@@ -95,7 +92,7 @@ static FT_Error  Load_Script( HB_Script*  s,
     cur_offset = FILE_Pos();
     if ( FILE_Seek( new_offset ) ||
 	 ( error = Load_LangSys( &s->DefaultLangSys,
-				 stream ) ) != FT_Err_Ok )
+				 stream ) ) != HB_Err_Ok )
       return error;
     (void)FILE_Seek( cur_offset );
   }
@@ -119,7 +116,7 @@ static FT_Error  Load_Script( HB_Script*  s,
 
   if ( s->LangSysCount == 0 && s->DefaultLangSys.FeatureCount == 0 )
   {
-    error = HB_Err_Empty_Script;
+    error = HB_Err_Not_Covered;
     goto Fail2;
   }
 
@@ -144,34 +141,33 @@ static FT_Error  Load_Script( HB_Script*  s,
 
     cur_offset = FILE_Pos();
     if ( FILE_Seek( new_offset ) ||
-	 ( error = Load_LangSys( &lsr[n].LangSys, stream ) ) != FT_Err_Ok )
+	 ( error = Load_LangSys( &lsr[n].LangSys, stream ) ) != HB_Err_Ok )
       goto Fail1;
     (void)FILE_Seek( cur_offset );
   }
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail1:
   for ( m = 0; m < n; m++ )
-    Free_LangSys( &lsr[m].LangSys, memory );
+    Free_LangSys( &lsr[m].LangSys );
 
   FREE( s->LangSysRecord );
 
 Fail2:
-  Free_LangSys( &s->DefaultLangSys, memory );
+  Free_LangSys( &s->DefaultLangSys );
   return error;
 }
 
 
-static void  Free_Script( HB_Script*  s,
-			  FT_Memory    memory )
+static void  Free_Script( HB_ScriptTable*  s )
 {
-  FT_UShort           n, count;
+  HB_UShort           n, count;
 
   HB_LangSysRecord*  lsr;
 
 
-  Free_LangSys( &s->DefaultLangSys, memory );
+  Free_LangSys( &s->DefaultLangSys );
 
   if ( s->LangSysRecord )
   {
@@ -179,7 +175,7 @@ static void  Free_Script( HB_Script*  s,
     lsr   = s->LangSysRecord;
 
     for ( n = 0; n < count; n++ )
-      Free_LangSys( &lsr[n].LangSys, memory );
+      Free_LangSys( &lsr[n].LangSys );
 
     FREE( lsr );
   }
@@ -188,14 +184,14 @@ static void  Free_Script( HB_Script*  s,
 
 /* ScriptList */
 
-FT_Error  _HB_OPEN_Load_ScriptList( HB_ScriptList*  sl,
-			   FT_Stream        stream )
+HB_INTERNAL HB_Error
+_HB_OPEN_Load_ScriptList( HB_ScriptList* sl,
+			  HB_Stream      stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort          n, script_count;
-  FT_ULong           cur_offset, new_offset, base_offset;
+  HB_UShort          n, script_count;
+  HB_UInt           cur_offset, new_offset, base_offset;
 
   HB_ScriptRecord*  sr;
 
@@ -233,9 +229,9 @@ FT_Error  _HB_OPEN_Load_ScriptList( HB_ScriptList*  sl,
       goto Fail;
 
     error = Load_Script( &sr[sl->ScriptCount].Script, stream );
-    if ( error == FT_Err_Ok )
+    if ( error == HB_Err_Ok )
       sl->ScriptCount += 1;
-    else if ( error != HB_Err_Empty_Script )
+    else if ( error != HB_Err_Not_Covered )
       goto Fail;
 
     (void)FILE_Seek( cur_offset );
@@ -247,26 +243,26 @@ FT_Error  _HB_OPEN_Load_ScriptList( HB_ScriptList*  sl,
 #if 0
   if ( sl->ScriptCount == 0 )
   {
-    error = HB_Err_Invalid_SubTable;
+    error = ERR(HB_Err_Invalid_SubTable);
     goto Fail;
   }
 #endif
   
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail:
   for ( n = 0; n < sl->ScriptCount; n++ )
-    Free_Script( &sr[n].Script, memory );
+    Free_Script( &sr[n].Script );
 
   FREE( sl->ScriptRecord );
   return error;
 }
 
 
-void  _HB_OPEN_Free_ScriptList( HB_ScriptList*  sl,
-		       FT_Memory        memory )
+HB_INTERNAL void
+_HB_OPEN_Free_ScriptList( HB_ScriptList* sl )
 {
-  FT_UShort          n, count;
+  HB_UShort          n, count;
 
   HB_ScriptRecord*  sr;
 
@@ -277,7 +273,7 @@ void  _HB_OPEN_Free_ScriptList( HB_ScriptList*  sl,
     sr    = sl->ScriptRecord;
 
     for ( n = 0; n < count; n++ )
-      Free_Script( &sr[n].Script, memory );
+      Free_Script( &sr[n].Script );
 
     FREE( sr );
   }
@@ -292,15 +288,14 @@ void  _HB_OPEN_Free_ScriptList( HB_ScriptList*  sl,
 
 /* Feature */
 
-static FT_Error  Load_Feature( HB_Feature*  f,
-			       FT_Stream     stream )
+static HB_Error  Load_Feature( HB_Feature*  f,
+			       HB_Stream     stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort   n, count;
+  HB_UShort   n, count;
 
-  FT_UShort*  lli;
+  HB_UShort*  lli;
 
 
   if ( ACCESS_Frame( 4L ) )
@@ -313,7 +308,7 @@ static FT_Error  Load_Feature( HB_Feature*  f,
 
   f->LookupListIndex = NULL;
 
-  if ( ALLOC_ARRAY( f->LookupListIndex, count, FT_UShort ) )
+  if ( ALLOC_ARRAY( f->LookupListIndex, count, HB_UShort ) )
     return error;
 
   lli = f->LookupListIndex;
@@ -329,12 +324,11 @@ static FT_Error  Load_Feature( HB_Feature*  f,
 
   FORGET_Frame();
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 }
 
 
-static void  Free_Feature( HB_Feature*  f,
-			   FT_Memory     memory )
+static void  Free_Feature( HB_Feature*  f )
 {
   FREE( f->LookupListIndex );
 }
@@ -342,14 +336,14 @@ static void  Free_Feature( HB_Feature*  f,
 
 /* FeatureList */
 
-FT_Error  _HB_OPEN_Load_FeatureList( HB_FeatureList*  fl,
-			    FT_Stream         stream )
+HB_INTERNAL HB_Error
+_HB_OPEN_Load_FeatureList( HB_FeatureList* fl,
+			   HB_Stream       stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort           n, m, count;
-  FT_ULong            cur_offset, new_offset, base_offset;
+  HB_UShort           n, m, count;
+  HB_UInt            cur_offset, new_offset, base_offset;
 
   HB_FeatureRecord*  fr;
 
@@ -367,7 +361,7 @@ FT_Error  _HB_OPEN_Load_FeatureList( HB_FeatureList*  fl,
 
   if ( ALLOC_ARRAY( fl->FeatureRecord, count, HB_FeatureRecord ) )
     return error;
-  if ( ALLOC_ARRAY( fl->ApplyOrder, count, FT_UShort ) )
+  if ( ALLOC_ARRAY( fl->ApplyOrder, count, HB_UShort ) )
     goto Fail2;
   
   fl->ApplyCount = 0;
@@ -386,16 +380,16 @@ FT_Error  _HB_OPEN_Load_FeatureList( HB_FeatureList*  fl,
 
     cur_offset = FILE_Pos();
     if ( FILE_Seek( new_offset ) ||
-	 ( error = Load_Feature( &fr[n].Feature, stream ) ) != FT_Err_Ok )
+	 ( error = Load_Feature( &fr[n].Feature, stream ) ) != HB_Err_Ok )
       goto Fail1;
     (void)FILE_Seek( cur_offset );
   }
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail1:
   for ( m = 0; m < n; m++ )
-    Free_Feature( &fr[m].Feature, memory );
+    Free_Feature( &fr[m].Feature );
 
   FREE( fl->ApplyOrder );
 
@@ -406,10 +400,10 @@ Fail2:
 }
 
 
-void  _HB_OPEN_Free_FeatureList( HB_FeatureList*  fl,
-			FT_Memory         memory)
+HB_INTERNAL void
+_HB_OPEN_Free_FeatureList( HB_FeatureList*  fl )
 {
-  FT_UShort           n, count;
+  HB_UShort           n, count;
 
   HB_FeatureRecord*  fr;
 
@@ -420,7 +414,7 @@ void  _HB_OPEN_Free_FeatureList( HB_FeatureList*  fl,
     fr    = fl->FeatureRecord;
 
     for ( n = 0; n < count; n++ )
-      Free_Feature( &fr[n].Feature, memory );
+      Free_Feature( &fr[n].Feature );
 
     FREE( fr );
   }
@@ -440,10 +434,10 @@ void  _HB_OPEN_Free_FeatureList( HB_FeatureList*  fl,
 
 /* SubTable */
 
-static FT_Error  Load_SubTable( HB_SubTable*  st,
-				FT_Stream     stream,
+static HB_Error  Load_SubTable( HB_SubTable*  st,
+				HB_Stream     stream,
 				HB_Type       table_type,
-				FT_UShort     lookup_type )
+				HB_UShort     lookup_type )
 {
   if ( table_type == HB_Type_GSUB )
     return _HB_GSUB_Load_SubTable ( &st->st.gsub, stream, lookup_type );
@@ -454,31 +448,29 @@ static FT_Error  Load_SubTable( HB_SubTable*  st,
 
 static void  Free_SubTable( HB_SubTable*  st,
 			    HB_Type       table_type,
-			    FT_UShort      lookup_type,
-			    FT_Memory      memory )
+			    HB_UShort     lookup_type )
 {
   if ( table_type == HB_Type_GSUB )
-    _HB_GSUB_Free_SubTable ( &st->st.gsub, memory, lookup_type );
+    _HB_GSUB_Free_SubTable ( &st->st.gsub, lookup_type );
   else
-    _HB_GPOS_Free_SubTable ( &st->st.gpos, memory, lookup_type );
+    _HB_GPOS_Free_SubTable ( &st->st.gpos, lookup_type );
 }
 
 
 /* Lookup */
 
-static FT_Error  Load_Lookup( HB_Lookup*   l,
-			      FT_Stream     stream,
+static HB_Error  Load_Lookup( HB_Lookup*   l,
+			      HB_Stream     stream,
 			      HB_Type      type )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort      n, m, count;
-  FT_ULong       cur_offset, new_offset, base_offset;
+  HB_UShort      n, m, count;
+  HB_UInt       cur_offset, new_offset, base_offset;
 
   HB_SubTable*  st;
 
-  FT_Bool        is_extension = FALSE;
+  HB_Bool        is_extension = FALSE;
 
 
   base_offset = FILE_Pos();
@@ -530,16 +522,16 @@ static FT_Error  Load_Lookup( HB_Lookup*   l,
 
     if ( FILE_Seek( new_offset ) ||
 	 ( error = Load_SubTable( &st[n], stream,
-				  type, l->LookupType ) ) != FT_Err_Ok )
+				  type, l->LookupType ) ) != HB_Err_Ok )
       goto Fail;
     (void)FILE_Seek( cur_offset );
   }
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail:
   for ( m = 0; m < n; m++ )
-    Free_SubTable( &st[m], type, l->LookupType, memory );
+    Free_SubTable( &st[m], type, l->LookupType );
 
   FREE( l->SubTable );
   return error;
@@ -547,10 +539,9 @@ Fail:
 
 
 static void  Free_Lookup( HB_Lookup*   l,
-			  HB_Type      type,
-			  FT_Memory     memory)
+			  HB_Type      type )
 {
-  FT_UShort      n, count;
+  HB_UShort      n, count;
 
   HB_SubTable*  st;
 
@@ -561,7 +552,7 @@ static void  Free_Lookup( HB_Lookup*   l,
     st    = l->SubTable;
 
     for ( n = 0; n < count; n++ )
-      Free_SubTable( &st[n], type, l->LookupType, memory );
+      Free_SubTable( &st[n], type, l->LookupType );
 
     FREE( st );
   }
@@ -570,15 +561,15 @@ static void  Free_Lookup( HB_Lookup*   l,
 
 /* LookupList */
 
-FT_Error  _HB_OPEN_Load_LookupList( HB_LookupList*  ll,
-			   FT_Stream        stream,
-			   HB_Type         type )
+HB_INTERNAL HB_Error
+_HB_OPEN_Load_LookupList( HB_LookupList* ll,
+			  HB_Stream      stream,
+			  HB_Type        type )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort    n, m, count;
-  FT_ULong     cur_offset, new_offset, base_offset;
+  HB_UShort    n, m, count;
+  HB_UInt     cur_offset, new_offset, base_offset;
 
   HB_Lookup*  l;
 
@@ -596,7 +587,7 @@ FT_Error  _HB_OPEN_Load_LookupList( HB_LookupList*  ll,
 
   if ( ALLOC_ARRAY( ll->Lookup, count, HB_Lookup ) )
     return error;
-  if ( ALLOC_ARRAY( ll->Properties, count, FT_UInt ) )
+  if ( ALLOC_ARRAY( ll->Properties, count, HB_UInt ) )
     goto Fail2;
 
   l = ll->Lookup;
@@ -612,18 +603,18 @@ FT_Error  _HB_OPEN_Load_LookupList( HB_LookupList*  ll,
 
     cur_offset = FILE_Pos();
     if ( FILE_Seek( new_offset ) ||
-	 ( error = Load_Lookup( &l[n], stream, type ) ) != FT_Err_Ok )
+	 ( error = Load_Lookup( &l[n], stream, type ) ) != HB_Err_Ok )
       goto Fail1;
     (void)FILE_Seek( cur_offset );
   }
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail1:
   FREE( ll->Properties );
 
   for ( m = 0; m < n; m++ )
-    Free_Lookup( &l[m], type, memory );
+    Free_Lookup( &l[m], type );
 
 Fail2:
   FREE( ll->Lookup );
@@ -631,11 +622,11 @@ Fail2:
 }
 
 
-void  _HB_OPEN_Free_LookupList( HB_LookupList*  ll,
-		       HB_Type         type,
-		       FT_Memory        memory )
+HB_INTERNAL void
+_HB_OPEN_Free_LookupList( HB_LookupList* ll,
+			  HB_Type        type )
 {
-  FT_UShort    n, count;
+  HB_UShort    n, count;
 
   HB_Lookup*  l;
 
@@ -648,7 +639,7 @@ void  _HB_OPEN_Free_LookupList( HB_LookupList*  ll,
     l     = ll->Lookup;
 
     for ( n = 0; n < count; n++ )
-      Free_Lookup( &l[n], type, memory );
+      Free_Lookup( &l[n], type );
 
     FREE( l );
   }
@@ -663,15 +654,14 @@ void  _HB_OPEN_Free_LookupList( HB_LookupList*  ll,
 
 /* CoverageFormat1 */
 
-static FT_Error  Load_Coverage1( HB_CoverageFormat1*  cf1,
-				 FT_Stream             stream )
+static HB_Error  Load_Coverage1( HB_CoverageFormat1*  cf1,
+				 HB_Stream             stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort  n, count;
+  HB_UShort  n, count;
 
-  FT_UShort* ga;
+  HB_UShort* ga;
 
 
   if ( ACCESS_Frame( 2L ) )
@@ -683,7 +673,7 @@ static FT_Error  Load_Coverage1( HB_CoverageFormat1*  cf1,
 
   cf1->GlyphArray = NULL;
 
-  if ( ALLOC_ARRAY( cf1->GlyphArray, count, FT_UShort ) )
+  if ( ALLOC_ARRAY( cf1->GlyphArray, count, HB_UShort ) )
     return error;
 
   ga = cf1->GlyphArray;
@@ -699,12 +689,11 @@ static FT_Error  Load_Coverage1( HB_CoverageFormat1*  cf1,
 
   FORGET_Frame();
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 }
 
 
-static void  Free_Coverage1( HB_CoverageFormat1*  cf1,
-			     FT_Memory             memory)
+static void  Free_Coverage1( HB_CoverageFormat1*  cf1 )
 {
   FREE( cf1->GlyphArray );
 }
@@ -712,13 +701,12 @@ static void  Free_Coverage1( HB_CoverageFormat1*  cf1,
 
 /* CoverageFormat2 */
 
-static FT_Error  Load_Coverage2( HB_CoverageFormat2*  cf2,
-				 FT_Stream             stream )
+static HB_Error  Load_Coverage2( HB_CoverageFormat2*  cf2,
+				 HB_Stream             stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort         n, count;
+  HB_UShort         n, count;
 
   HB_RangeRecord*  rr;
 
@@ -751,14 +739,14 @@ static FT_Error  Load_Coverage2( HB_CoverageFormat2*  cf2,
 	 ( rr[n].End - rr[n].Start + (long)rr[n].StartCoverageIndex ) >=
 	   0x10000L )
     {
-      error = HB_Err_Invalid_SubTable;
+      error = ERR(HB_Err_Invalid_SubTable);
       goto Fail;
     }
   }
 
   FORGET_Frame();
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail:
   FREE( cf2->RangeRecord );
@@ -766,17 +754,17 @@ Fail:
 }
 
 
-static void  Free_Coverage2( HB_CoverageFormat2*  cf2,
-			     FT_Memory             memory )
+static void  Free_Coverage2( HB_CoverageFormat2*  cf2 )
 {
   FREE( cf2->RangeRecord );
 }
 
 
-FT_Error  _HB_OPEN_Load_Coverage( HB_Coverage*  c,
-			 FT_Stream      stream )
+HB_INTERNAL HB_Error
+_HB_OPEN_Load_Coverage( HB_Coverage* c,
+			HB_Stream    stream )
 {
-  FT_Error   error;
+  HB_Error   error;
 
   if ( ACCESS_Frame( 2L ) )
     return error;
@@ -787,43 +775,34 @@ FT_Error  _HB_OPEN_Load_Coverage( HB_Coverage*  c,
 
   switch ( c->CoverageFormat )
   {
-  case 1:
-    return Load_Coverage1( &c->cf.cf1, stream );
-
-  case 2:
-    return Load_Coverage2( &c->cf.cf2, stream );
-
-  default:
-    return HB_Err_Invalid_SubTable_Format;
+  case 1:  return Load_Coverage1( &c->cf.cf1, stream );
+  case 2:  return Load_Coverage2( &c->cf.cf2, stream );
+  default: return ERR(HB_Err_Invalid_SubTable_Format);
   }
 
-  return FT_Err_Ok;               /* never reached */
+  return HB_Err_Ok;               /* never reached */
 }
 
 
-void  _HB_OPEN_Free_Coverage( HB_Coverage*  c,
-		     FT_Memory      memory )
+HB_INTERNAL void
+_HB_OPEN_Free_Coverage( HB_Coverage* c )
 {
   switch ( c->CoverageFormat )
   {
-  case 1:
-    Free_Coverage1( &c->cf.cf1, memory );
-    break;
-
-  case 2:
-    Free_Coverage2( &c->cf.cf2, memory );
-    break;
+  case 1:  Free_Coverage1( &c->cf.cf1 ); break;
+  case 2:  Free_Coverage2( &c->cf.cf2 ); break;
+  default:					 break;
   }
 }
 
 
-static FT_Error  Coverage_Index1( HB_CoverageFormat1*  cf1,
-				  FT_UShort             glyphID,
-				  FT_UShort*            index )
+static HB_Error  Coverage_Index1( HB_CoverageFormat1*  cf1,
+				  HB_UShort             glyphID,
+				  HB_UShort*            index )
 {
-  FT_UShort min, max, new_min, new_max, middle;
+  HB_UShort min, max, new_min, new_max, middle;
 
-  FT_UShort*  array = cf1->GlyphArray;
+  HB_UShort*  array = cf1->GlyphArray;
 
 
   /* binary search */
@@ -847,7 +826,7 @@ static FT_Error  Coverage_Index1( HB_CoverageFormat1*  cf1,
     if ( glyphID == array[middle] )
     {
       *index = middle;
-      return FT_Err_Ok;
+      return HB_Err_Ok;
     }
     else if ( glyphID < array[middle] )
     {
@@ -867,11 +846,11 @@ static FT_Error  Coverage_Index1( HB_CoverageFormat1*  cf1,
 }
 
 
-static FT_Error  Coverage_Index2( HB_CoverageFormat2*  cf2,
-				  FT_UShort             glyphID,
-				  FT_UShort*            index )
+static HB_Error  Coverage_Index2( HB_CoverageFormat2*  cf2,
+				  HB_UShort             glyphID,
+				  HB_UShort*            index )
 {
-  FT_UShort         min, max, new_min, new_max, middle;
+  HB_UShort         min, max, new_min, new_max, middle;
 
   HB_RangeRecord*  rr = cf2->RangeRecord;
 
@@ -897,7 +876,7 @@ static FT_Error  Coverage_Index2( HB_CoverageFormat2*  cf2,
     if ( glyphID >= rr[middle].Start && glyphID <= rr[middle].End )
     {
       *index = rr[middle].StartCoverageIndex + glyphID - rr[middle].Start;
-      return FT_Err_Ok;
+      return HB_Err_Ok;
     }
     else if ( glyphID < rr[middle].Start )
     {
@@ -917,23 +896,19 @@ static FT_Error  Coverage_Index2( HB_CoverageFormat2*  cf2,
 }
 
 
-FT_Error  _HB_OPEN_Coverage_Index( HB_Coverage*  c,
-			  FT_UShort      glyphID,
-			  FT_UShort*     index )
+HB_INTERNAL HB_Error
+_HB_OPEN_Coverage_Index( HB_Coverage* c,
+			 HB_UShort    glyphID,
+			 HB_UShort*   index )
 {
   switch ( c->CoverageFormat )
   {
-  case 1:
-    return Coverage_Index1( &c->cf.cf1, glyphID, index );
-
-  case 2:
-    return Coverage_Index2( &c->cf.cf2, glyphID, index );
-
-  default:
-    return HB_Err_Invalid_SubTable_Format;
+  case 1:  return Coverage_Index1( &c->cf.cf1, glyphID, index );
+  case 2:  return Coverage_Index2( &c->cf.cf2, glyphID, index );
+  default: return ERR(HB_Err_Invalid_SubTable_Format);
   }
 
-  return FT_Err_Ok;               /* never reached */
+  return HB_Err_Ok;               /* never reached */
 }
 
 
@@ -945,17 +920,16 @@ FT_Error  _HB_OPEN_Coverage_Index( HB_Coverage*  c,
 
 /* ClassDefFormat1 */
 
-static FT_Error  Load_ClassDef1( HB_ClassDefinition*  cd,
-				 FT_UShort             limit,
-				 FT_Stream             stream )
+static HB_Error  Load_ClassDef1( HB_ClassDefinition*  cd,
+				 HB_UShort             limit,
+				 HB_Stream             stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort             n, count;
+  HB_UShort             n, count;
 
-  FT_UShort*            cva;
-  FT_Bool*              d;
+  HB_UShort*            cva;
+  HB_Bool*              d;
 
   HB_ClassDefFormat1*  cdf1;
 
@@ -973,11 +947,11 @@ static FT_Error  Load_ClassDef1( HB_ClassDefinition*  cd,
   /* sanity check; we are limited to 16bit integers */
 
   if ( cdf1->StartGlyph + (long)count >= 0x10000L )
-    return HB_Err_Invalid_SubTable;
+    return ERR(HB_Err_Invalid_SubTable);
 
   cdf1->ClassValueArray = NULL;
 
-  if ( ALLOC_ARRAY( cdf1->ClassValueArray, count, FT_UShort ) )
+  if ( ALLOC_ARRAY( cdf1->ClassValueArray, count, HB_UShort ) )
     return error;
 
   d   = cd->Defined;
@@ -991,7 +965,7 @@ static FT_Error  Load_ClassDef1( HB_ClassDefinition*  cd,
     cva[n] = GET_UShort();
     if ( cva[n] >= limit )
     {
-      error = HB_Err_Invalid_SubTable;
+      error = ERR(HB_Err_Invalid_SubTable);
       goto Fail;
     }
     d[cva[n]] = TRUE;
@@ -999,7 +973,7 @@ static FT_Error  Load_ClassDef1( HB_ClassDefinition*  cd,
 
   FORGET_Frame();
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail:
   FREE( cva );
@@ -1008,8 +982,7 @@ Fail:
 }
 
 
-static void  Free_ClassDef1( HB_ClassDefFormat1*  cdf1,
-			     FT_Memory             memory )
+static void  Free_ClassDef1( HB_ClassDefFormat1*  cdf1 )
 {
   FREE( cdf1->ClassValueArray );
 }
@@ -1017,17 +990,16 @@ static void  Free_ClassDef1( HB_ClassDefFormat1*  cdf1,
 
 /* ClassDefFormat2 */
 
-static FT_Error  Load_ClassDef2( HB_ClassDefinition*  cd,
-				 FT_UShort             limit,
-				 FT_Stream             stream )
+static HB_Error  Load_ClassDef2( HB_ClassDefinition*  cd,
+				 HB_UShort             limit,
+				 HB_Stream             stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort              n, count;
+  HB_UShort              n, count;
 
   HB_ClassRangeRecord*  crr;
-  FT_Bool*               d;
+  HB_Bool*               d;
 
   HB_ClassDefFormat2*   cdf2;
 
@@ -1037,7 +1009,8 @@ static FT_Error  Load_ClassDef2( HB_ClassDefinition*  cd,
   if ( ACCESS_Frame( 2L ) )
     return error;
 
-  count = cdf2->ClassRangeCount = GET_UShort();
+  count = GET_UShort();
+  cdf2->ClassRangeCount = 0; /* zero for now.  we fill with the number of good entries later */
 
   FORGET_Frame();
 
@@ -1063,15 +1036,22 @@ static FT_Error  Load_ClassDef2( HB_ClassDefinition*  cd,
     if ( crr[n].Start > crr[n].End ||
 	 crr[n].Class >= limit )
     {
-      error = HB_Err_Invalid_SubTable;
-      goto Fail;
+      /* XXX
+       * Corrupt entry.  Skip it.
+       * This is hit by Nafees Nastaliq font for example
+       */
+       n--;
+       count--;
     }
-    d[crr[n].Class] = TRUE;
+    else
+      d[crr[n].Class] = TRUE;
   }
 
   FORGET_Frame();
 
-  return FT_Err_Ok;
+  cdf2->ClassRangeCount = count;
+
+  return HB_Err_Ok;
 
 Fail:
   FREE( crr );
@@ -1080,8 +1060,7 @@ Fail:
 }
 
 
-static void  Free_ClassDef2( HB_ClassDefFormat2*  cdf2,
-			     FT_Memory             memory )
+static void  Free_ClassDef2( HB_ClassDefFormat2*  cdf2 )
 {
   FREE( cdf2->ClassRangeRecord );
 }
@@ -1089,15 +1068,14 @@ static void  Free_ClassDef2( HB_ClassDefFormat2*  cdf2,
 
 /* ClassDefinition */
 
-FT_Error  _HB_OPEN_Load_ClassDefinition( HB_ClassDefinition*  cd,
-				FT_UShort             limit,
-				FT_Stream             stream )
+HB_INTERNAL HB_Error
+_HB_OPEN_Load_ClassDefinition( HB_ClassDefinition* cd,
+			       HB_UShort           limit,
+			       HB_Stream           stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-
-  if ( ALLOC_ARRAY( cd->Defined, limit, FT_Bool ) )
+  if ( ALLOC_ARRAY( cd->Defined, limit, HB_Bool ) )
     return error;
 
   if ( ACCESS_Frame( 2L ) )
@@ -1109,17 +1087,9 @@ FT_Error  _HB_OPEN_Load_ClassDefinition( HB_ClassDefinition*  cd,
 
   switch ( cd->ClassFormat )
   {
-  case 1:
-    error = Load_ClassDef1( cd, limit, stream );
-    break;
-
-  case 2:
-    error = Load_ClassDef2( cd, limit, stream );
-    break;
-
-  default:
-    error = HB_Err_Invalid_SubTable_Format;
-    break;
+  case 1:  error = Load_ClassDef1( cd, limit, stream ); break;
+  case 2:  error = Load_ClassDef2( cd, limit, stream ); break;
+  default: error = ERR(HB_Err_Invalid_SubTable_Format);	break;
   }
 
   if ( error )
@@ -1127,7 +1097,7 @@ FT_Error  _HB_OPEN_Load_ClassDefinition( HB_ClassDefinition*  cd,
 
   cd->loaded = TRUE;
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail:
   FREE( cd->Defined );
@@ -1135,31 +1105,55 @@ Fail:
 }
 
 
-FT_Error  _HB_OPEN_Load_EmptyClassDefinition( HB_ClassDefinition*  cd,
-				     FT_Stream             stream )
+static HB_Error
+_HB_OPEN_Load_EmptyClassDefinition( HB_ClassDefinition*  cd )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-
-  if ( ALLOC_ARRAY( cd->Defined, 1, FT_Bool ) )
+  if ( ALLOC_ARRAY( cd->Defined, 1, HB_Bool ) )
     return error;
 
   cd->ClassFormat = 1; /* Meaningless */
   cd->Defined[0] = FALSE;
 
-  if ( ALLOC_ARRAY( cd->cd.cd1.ClassValueArray, 1, FT_UShort ) )
+  if ( ALLOC_ARRAY( cd->cd.cd1.ClassValueArray, 1, HB_UShort ) )
     goto Fail;
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 
 Fail:
   FREE( cd->Defined );
   return error;
 }
 
-void  _HB_OPEN_Free_ClassDefinition( HB_ClassDefinition*  cd,
-			    FT_Memory             memory )
+HB_INTERNAL HB_Error
+_HB_OPEN_Load_EmptyOrClassDefinition( HB_ClassDefinition* cd,
+				      HB_UShort           limit,
+				      HB_UInt            class_offset,
+				      HB_UInt            base_offset,
+				      HB_Stream           stream )
+{
+  HB_Error error;
+  HB_UInt               cur_offset;
+
+  cur_offset = FILE_Pos();
+
+  if ( class_offset )
+    {
+      if ( !FILE_Seek( class_offset + base_offset ) )
+	error = _HB_OPEN_Load_ClassDefinition( cd, limit, stream );
+    }
+  else
+     error = _HB_OPEN_Load_EmptyClassDefinition ( cd );
+
+  if (error == HB_Err_Ok)
+    (void)FILE_Seek( cur_offset ); /* Changes error as a side-effect */
+
+  return error;
+}
+
+HB_INTERNAL void
+_HB_OPEN_Free_ClassDefinition( HB_ClassDefinition*  cd )
 {
   if ( !cd->loaded )
     return;
@@ -1168,37 +1162,33 @@ void  _HB_OPEN_Free_ClassDefinition( HB_ClassDefinition*  cd,
 
   switch ( cd->ClassFormat )
   {
-  case 1:
-    Free_ClassDef1( &cd->cd.cd1, memory );
-    break;
-
-  case 2:
-    Free_ClassDef2( &cd->cd.cd2, memory );
-    break;
+  case 1:  Free_ClassDef1( &cd->cd.cd1 ); break;
+  case 2:  Free_ClassDef2( &cd->cd.cd2 ); break;
+  default:				  break;
   }
 }
 
 
-static FT_Error  Get_Class1( HB_ClassDefFormat1*  cdf1,
-			     FT_UShort             glyphID,
-			     FT_UShort*            class,
-			     FT_UShort*            index )
+static HB_Error  Get_Class1( HB_ClassDefFormat1*  cdf1,
+			     HB_UShort             glyphID,
+			     HB_UShort*            klass,
+			     HB_UShort*            index )
 {
-  FT_UShort*  cva = cdf1->ClassValueArray;
+  HB_UShort*  cva = cdf1->ClassValueArray;
 
 
   if ( index )
     *index = 0;
 
   if ( glyphID >= cdf1->StartGlyph &&
-       glyphID <= cdf1->StartGlyph + cdf1->GlyphCount )
+       glyphID < cdf1->StartGlyph + cdf1->GlyphCount )
   {
-    *class = cva[glyphID - cdf1->StartGlyph];
-    return FT_Err_Ok;
+    *klass = cva[glyphID - cdf1->StartGlyph];
+    return HB_Err_Ok;
   }
   else
   {
-    *class = 0;
+    *klass = 0;
     return HB_Err_Not_Covered;
   }
 }
@@ -1207,13 +1197,13 @@ static FT_Error  Get_Class1( HB_ClassDefFormat1*  cdf1,
 /* we need the index value of the last searched class range record
    in case of failure for constructed GDEF tables                  */
 
-static FT_Error  Get_Class2( HB_ClassDefFormat2*  cdf2,
-			     FT_UShort             glyphID,
-			     FT_UShort*            class,
-			     FT_UShort*            index )
+static HB_Error  Get_Class2( HB_ClassDefFormat2*  cdf2,
+			     HB_UShort             glyphID,
+			     HB_UShort*            klass,
+			     HB_UShort*            index )
 {
-  FT_Error               error = FT_Err_Ok;
-  FT_UShort              min, max, new_min, new_max, middle;
+  HB_Error               error = HB_Err_Ok;
+  HB_UShort              min, max, new_min, new_max, middle;
 
   HB_ClassRangeRecord*  crr = cdf2->ClassRangeRecord;
 
@@ -1222,7 +1212,7 @@ static FT_Error  Get_Class2( HB_ClassDefFormat2*  cdf2,
 
   if ( cdf2->ClassRangeCount == 0 )
     {
-      *class = 0;
+      *klass = 0;
       if ( index )
 	*index = 0;
       
@@ -1244,15 +1234,15 @@ static FT_Error  Get_Class2( HB_ClassDefFormat2*  cdf2,
 
     if ( glyphID >= crr[middle].Start && glyphID <= crr[middle].End )
     {
-      *class = crr[middle].Class;
-      error  = FT_Err_Ok;
+      *klass = crr[middle].Class;
+      error  = HB_Err_Ok;
       break;
     }
     else if ( glyphID < crr[middle].Start )
     {
       if ( middle == min )
       {
-	*class = 0;
+	*klass = 0;
 	error  = HB_Err_Not_Covered;
 	break;
       }
@@ -1262,7 +1252,7 @@ static FT_Error  Get_Class2( HB_ClassDefFormat2*  cdf2,
     {
       if ( middle == max )
       {
-	*class = 0;
+	*klass = 0;
 	error  = HB_Err_Not_Covered;
 	break;
       }
@@ -1277,24 +1267,20 @@ static FT_Error  Get_Class2( HB_ClassDefFormat2*  cdf2,
 }
 
 
-FT_Error  _HB_OPEN_Get_Class( HB_ClassDefinition*  cd,
-		     FT_UShort             glyphID,
-		     FT_UShort*            class,
-		     FT_UShort*            index )
+HB_INTERNAL HB_Error
+_HB_OPEN_Get_Class( HB_ClassDefinition* cd,
+		    HB_UShort           glyphID,
+		    HB_UShort*          klass,
+		    HB_UShort*          index )
 {
   switch ( cd->ClassFormat )
   {
-  case 1:
-    return Get_Class1( &cd->cd.cd1, glyphID, class, index );
-
-  case 2:
-    return Get_Class2( &cd->cd.cd2, glyphID, class, index );
-
-  default:
-    return HB_Err_Invalid_SubTable_Format;
+  case 1:  return Get_Class1( &cd->cd.cd1, glyphID, klass, index );
+  case 2:  return Get_Class2( &cd->cd.cd2, glyphID, klass, index );
+  default: return ERR(HB_Err_Invalid_SubTable_Format);
   }
 
-  return FT_Err_Ok;               /* never reached */
+  return HB_Err_Ok;               /* never reached */
 }
 
 
@@ -1304,15 +1290,15 @@ FT_Error  _HB_OPEN_Get_Class( HB_ClassDefinition*  cd,
  ***************************/
 
 
-FT_Error  _HB_OPEN_Load_Device( HB_Device*  d,
-		       FT_Stream    stream )
+HB_INTERNAL HB_Error
+_HB_OPEN_Load_Device( HB_Device* d,
+		      HB_Stream  stream )
 {
-  FT_Error   error;
-  FT_Memory  memory = stream->memory;
+  HB_Error   error;
 
-  FT_UShort   n, count;
+  HB_UShort   n, count;
 
-  FT_UShort*  dv;
+  HB_UShort*  dv;
 
 
   if ( ACCESS_Frame( 6L ) )
@@ -1324,16 +1310,23 @@ FT_Error  _HB_OPEN_Load_Device( HB_Device*  d,
 
   FORGET_Frame();
 
+  d->DeltaValue = NULL;
+
   if ( d->StartSize > d->EndSize ||
        d->DeltaFormat == 0 || d->DeltaFormat > 3 )
-    return HB_Err_Invalid_SubTable;
-
-  d->DeltaValue = NULL;
+    {
+      /* XXX
+       * I've seen fontforge generate DeltaFormat == 0.
+       * Just return Ok and let the NULL DeltaValue disable
+       * this table.
+       */
+      return HB_Err_Ok;
+    }
 
   count = ( ( d->EndSize - d->StartSize + 1 ) >>
 	      ( 4 - d->DeltaFormat ) ) + 1;
 
-  if ( ALLOC_ARRAY( d->DeltaValue, count, FT_UShort ) )
+  if ( ALLOC_ARRAY( d->DeltaValue, count, HB_UShort ) )
     return error;
 
   if ( ACCESS_Frame( count * 2L ) )
@@ -1349,12 +1342,12 @@ FT_Error  _HB_OPEN_Load_Device( HB_Device*  d,
 
   FORGET_Frame();
 
-  return FT_Err_Ok;
+  return HB_Err_Ok;
 }
 
 
-void  _HB_OPEN_Free_Device( HB_Device*  d,
-		   FT_Memory    memory )
+HB_INTERNAL void
+_HB_OPEN_Free_Device( HB_Device* d )
 {
   FREE( d->DeltaValue );
 }
@@ -1395,11 +1388,12 @@ void  _HB_OPEN_Free_Device( HB_Device*  d,
 
      mask = 0x00FF                                    */
 
-FT_Error  _HB_OPEN_Get_Device( HB_Device*  d,
-		      FT_UShort    size,
-		      FT_Short*    value )
+HB_INTERNAL HB_Error
+_HB_OPEN_Get_Device( HB_Device* d,
+		     HB_UShort  size,
+		     HB_Short*  value )
 {
-  FT_UShort  byte, bits, mask, f, s;
+  HB_UShort  byte, bits, mask, f, s;
 
 
   f = d->DeltaFormat;
@@ -1411,14 +1405,14 @@ FT_Error  _HB_OPEN_Get_Device( HB_Device*  d,
     bits = byte >> ( 16 - ( ( s % ( 1 << ( 4 - f ) ) + 1 ) << f ) );
     mask = 0xFFFF >> ( 16 - ( 1 << f ) );
 
-    *value = (FT_Short)( bits & mask );
+    *value = (HB_Short)( bits & mask );
 
     /* conversion to a signed value */
 
     if ( *value >= ( ( mask + 1 ) >> 1 ) )
       *value -= mask + 1;
 
-    return FT_Err_Ok;
+    return HB_Err_Ok;
   }
   else
   {
