@@ -10,10 +10,10 @@
  *  See the file name COPYING for licensing information.
  *
  ******************************************************************/
-#include "ftglue.c"
-#include "harfbuzz-open.c"
+#define HB_INTERNAL static
 #include "harfbuzz-buffer.c"
 #include "harfbuzz-gdef.c"
 #include "harfbuzz-gsub.c"
 #include "harfbuzz-gpos.c"
-#include "harfbuzz-dump.c"
+#include "harfbuzz-impl.c"
+#include "harfbuzz-open.c"
